package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.util.CollectionUtils;
import pps.cloud.base.service.BaseBasePipelineService;
import pps.cloud.base.service.BaseBaseStationService;
import pps.cloud.base.service.data.BaseOuIdInput;
import pps.cloud.base.service.data.QueryBaseStationOutput;
import pps.cloud.base.service.data.base_pipeline.SelectPipelineByOperatorOutput;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.data.GetSysOrganizationViewOutput;
import pps.core.base.entity.BaseDataEnt;
import pps.core.base.entity.BaseDataView;
import pps.core.base.enums.ImportFieldDic;
import pps.core.base.mapper.BaseDataMapper;
import pps.core.base.mapper.BaseDataViewMapper;
import pps.core.base.service.data.base_data.*;
import pps.core.base.service.data.base_line.ImportFileInput;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.ExcelTemplateUtils;
import pps.core.common.utils.ExcelUtils;
import pps.core.common.utils.ExportExcelUtils;
import pps.core.common.utils.MapUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.tool.XStorageTool;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XDateUtils;
import xstartup.data.*;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiUpload;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

@XService
public class BaseDataService {
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBaseData(XContext context, CreateBaseDataInput input){
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        BaseDataEnt entity = new BaseDataEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBaseData(XContext context, UpdateBaseDataInput input){
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        QueryWrapper<BaseDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseDataEnt::getId, input.getId());
        BaseDataEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBaseData(XContext context, DeleteBaseDataInput input){
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        QueryWrapper<BaseDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseDataEnt::getId, input.getId());
        BaseDataEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBaseDataOutput> getBaseData(XContext context, GetBaseDataInput input){
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        QueryWrapper<BaseDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseDataEnt::getId, input.getId());
        BaseDataEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseDataOutput output = new GetBaseDataOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseDataOutput> queryBaseData(XContext context, QueryBaseDataInput input){
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        QueryWrapper<BaseDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseDataEnt::getOuId, input.getOuId())
            .eq(BaseDataEnt::getLineId, input.getLineId())
            .eq(BaseDataEnt::getDataDate, input.getDataDate());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseDataOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseDataOutput> dynamicQueryBaseData(XContext context, CustomQueryInput input){
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        return XMapperHelper.query(mapper, input, BaseDataEnt.class, QueryBaseDataOutput.class);
    }


    @XText("下载导入模板")
    @XApiGet
    public XFileResult downloadImportTemplate(XContext context){
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/导入模板_" + XDateUtils.getString(new Date(),"yyyy-MM-dd") + ".xlsx");
        String templateFilePath = "template/import_template.xlsx";
        InputStream templateInputStream = this.getClass().getClassLoader().getResourceAsStream(templateFilePath);
        byte[] excelContent = ExportExcelUtils.genSingleExcelFileData(templateInputStream, null);
        try {
            ExportExcelUtils.outputFileData(excelContent,fileSavePath);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
    }

    @XText("新增导入")
    @XApiUpload
    public XServiceResult importFile(XContext context, ImportFileInput input) {
        List<String> headerList = getExcelHeaderList();
        List<BaseDataView> list = new ArrayList<>();
        List<Map<String, Object>> mapList = ExcelUtils.readExcel(input.getFile().getInputStream(), input.getFile().getFileName(), headerList, 0);

        try {
            List<BaseDataView> dataList = new ArrayList<>();
            Date minDate = null;
            Date maxDate = null;
            for (Map<String, Object> objectMap : mapList) {
                BaseDataView dataView = MapUtil.mapToObj(objectMap, BaseDataView.class);
                dataView.setOuId(input.getOuId());
                dataView.setLineId(input.getLineId());
                if(dataView.getDataDate()!=null){
                    if(minDate==null){
                        minDate = dataView.getDataDate();
                        maxDate = dataView.getDataDate();
                    }
                    if(minDate.compareTo(dataView.getDataDate())>0){
                        minDate = dataView.getDataDate();
                    }else if(maxDate.compareTo(dataView.getDataDate())<0){
                        maxDate = dataView.getDataDate();
                    }
                }
                dataList.add(dataView);
            }
            if(minDate==null){
                return XServiceResult.error(1000,"日期错误");
            }
            BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
            QueryWrapper<BaseDataEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BaseDataEnt::getLineId,input.getLineId())
                    .ge(BaseDataEnt::getDataDate,minDate)
                    .le(BaseDataEnt::getDataDate,maxDate);
            mapper.delete(queryWrapper);
            BaseDataViewMapper viewMapper = context.getBean(BaseDataViewMapper.class);
            viewMapper.batchInsert(dataList);
        }catch (Exception e){
            return XServiceResult.error(-1,e.getMessage());
        }
        return XServiceResult.OK;
    }

    private List<String> getExcelHeaderList(){
        List<Map<String,Object>> fieldList = new ArrayList<>();
        for (ImportFieldDic value : ImportFieldDic.values()) {
            Map<String, Object> map = new HashMap<String, Object>();
            map.put("col",value.getCol());
            map.put("sort",value.getSort());
            fieldList.add(map);
        }
        fieldList.sort(new Comparator<Map<String, Object>>() {
            @Override
            public int compare(Map<String, Object> o1, Map<String, Object> o2) {
                return ((Integer)o1.get("sort")).compareTo((Integer)o2.get("sort"));
            }
        });
        return fieldList.stream().map(x->((String)x.get("col"))).collect(Collectors.toList());
    }
}
